package org.greenstone.android.tipple.base;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.greenstone.android.tipple.R;
import org.mapsforge.android.maps.ArrayItemizedTextAudioOverlay;
import org.mapsforge.android.maps.CirclesOverlay;
import org.mapsforge.android.maps.GeoPoint;
import org.mapsforge.android.maps.MapView;
import org.mapsforge.android.maps.Overlay;
import org.mapsforge.android.maps.OverlayItem;
import org.mapsforge.android.maps.OverlayTextAudioItem;
import org.mapsforge.android.maps.RouteOverlayResetable;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import android.app.Activity;
import android.graphics.Color;
import android.graphics.DashPathEffect;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnUtteranceCompletedListener;
import android.util.Log;

public class TipLocationManager 
{
	Activity activity_;
	AudioPlayerView audio_player_view_;
	TextToSpeech tts_;
	
	public static CirclesOverlay circlesOverlay=null; 
	public static ArrayItemizedTextAudioOverlay itemizedMarkerOverlay;
	public static ArrayList<OverlayItem> itemizedMarkerOverlayArrayList;
	public static ArrayList<GeoPoint> circlesOverlayGeoPoints;
	public static ArrayList<Integer> circlesOverlayRadii;
	
	public static RouteOverlayResetable routeOverlay=null;
	public static ArrayItemizedTextAudioOverlay routeMarkerOverlay;
	public static GeoPoint[] routeOverlayArray;
	
	public static ArrayList<OverlayItem> routeMarkerOverlayArrayList;
	
	public static String prevVisitedFilename;
	
	TipLocationManager(Activity activity, AudioPlayerView audio_player_view, 
						 TextToSpeech tts)
	{
		activity_ = activity;
		audio_player_view_ = audio_player_view;
		tts_ = tts;
	
	
		
	}
	
	protected void createLogRouteLocationsOverlay(MapView mapView,ArrayList<TipLocation> locations)
	{
		Paint fillPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		fillPaint.setStyle(Paint.Style.STROKE);
		fillPaint.setColor(Color.BLUE);
		fillPaint.setAlpha(160);
		fillPaint.setStrokeWidth(7);
		fillPaint.setStrokeCap(Paint.Cap.BUTT);
		fillPaint.setStrokeJoin(Paint.Join.ROUND);
		fillPaint.setPathEffect(new DashPathEffect(new float[] { 20, 20 }, 0));

		Paint outlinePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		outlinePaint.setStyle(Paint.Style.STROKE);
		outlinePaint.setColor(Color.BLUE);
		outlinePaint.setAlpha(96);
		outlinePaint.setStrokeWidth(7);
		outlinePaint.setStrokeCap(Paint.Cap.BUTT);
		outlinePaint.setStrokeJoin(Paint.Join.ROUND);
	
		Drawable defaultMarker = activity_.getResources().getDrawable(R.drawable.routemarker);
		
		if (routeOverlay==null) {
			routeOverlay = new RouteOverlayResetable(fillPaint, outlinePaint);
			routeMarkerOverlay = new ArrayItemizedTextAudioOverlay(defaultMarker,activity_,tts_);
			routeMarkerOverlayArrayList = new ArrayList<OverlayItem>();
		
		}
		// This one needs to be done again, incase it is of a different size
		routeOverlayArray = new GeoPoint[locations.size()];
		
		for (int i=0;  i<locations.size(); i++) {
			TipLocation location = locations.get(i);
			double latitude  = location.getLatitude();
			double longitude = location.getLongitude();
			String title     = location.getTitle();
			String text      = location.getText();
			String tts_text  = location.getTTSText();
			GeoPoint geoPoint = new GeoPoint(latitude,longitude);
				
			routeOverlayArray[i] = geoPoint;
			
			if (title.startsWith("Manual")) {
				// ItemizedOverlay => display marker with text for info pop-up window
				OverlayTextAudioItem overlayItem = new OverlayTextAudioItem(geoPoint, title, text, tts_text);
				
				//routeMarkerOverlay.addOverlay(overlayItem);
				routeMarkerOverlayArrayList.add(overlayItem);
			}
		}
		
		//routeOverlay.setRouteData(routeOverlayArray);
		
		// Only add in if not already present
		List<Overlay> overlays = mapView.getOverlays();
		if (!overlays.contains(routeOverlay)) {
			overlays.add(routeOverlay);
		}
		if (!overlays.contains(routeMarkerOverlay)) {
			overlays.add(routeMarkerOverlay);
		}
	}
	
	protected void addLogRouteLocations(MapView mapView)
	{	
		for (int i=0; i<routeMarkerOverlayArrayList.size(); i++) {
			
			OverlayItem overlay_item = routeMarkerOverlayArrayList.get(i);
			routeMarkerOverlay.addOverlay(overlay_item);
		}
	
		routeOverlay.setRouteData(routeOverlayArray);
		
		mapView.invalidate();
		
	}
	
	protected void removeLogRouteLocations(MapView mapView)
	{		
		for (int i=0; i<routeMarkerOverlayArrayList.size(); i++) {
			
			OverlayItem overlay_item = routeMarkerOverlayArrayList.get(i);
			routeMarkerOverlay.removeOverlay(overlay_item);
			
		}
		
		routeOverlay.reset();
		
		mapView.invalidate();
	}
	
	protected void createTourLocationsOverlay(MapView mapView, ArrayList<TipLocation> locations)
	{
		Paint circleFillPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		circleFillPaint.setStyle(Paint.Style.FILL);
		circleFillPaint.setColor(Color.BLUE);
		circleFillPaint.setAlpha(64);

		Paint circleOutlinePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		circleOutlinePaint.setStyle(Paint.Style.STROKE);
		circleOutlinePaint.setColor(Color.BLUE);
		circleOutlinePaint.setAlpha(128);
		circleOutlinePaint.setStrokeWidth(3);
	
		Drawable defaultMarker = activity_.getResources().getDrawable(R.drawable.marker);
		
		if (circlesOverlay==null) {
			circlesOverlay          = new CirclesOverlay(circleFillPaint, circleOutlinePaint);
			circlesOverlayGeoPoints = new ArrayList<GeoPoint>();
			circlesOverlayRadii     = new ArrayList<Integer>();
		
			itemizedMarkerOverlay = new ArrayItemizedTextAudioOverlay(defaultMarker,activity_,tts_);
			itemizedMarkerOverlayArrayList = new ArrayList<OverlayItem>();
		}
		
		for (int i=0;  i<locations.size(); i++) {
			TipLocation location = locations.get(i);
			double latitude  = location.getLatitude();
			double longitude = location.getLongitude();
			int radius       = (int)Math.round(1000.0 * location.getRadiusKM());
			String title     = location.getTitle();
			String text      = location.getText();
			String tts_text  = location.getTTSText();
			GeoPoint geoPoint = new GeoPoint(latitude,longitude);
				
			// CirclesOverlay => Highlight location with a circle showing catchment radius
			//circlesOverlay.addCircleData(geoPoint, radius);
			circlesOverlayGeoPoints.add(geoPoint);
			circlesOverlayRadii.add(radius);
			
			// ItemizedOverlay => display marker with text for info pop-up window
			OverlayTextAudioItem overlayItem = new OverlayTextAudioItem(geoPoint, title, text, tts_text);
			 
			//itemizedMarkerOverlay.addOverlay(overlayItem);
			itemizedMarkerOverlayArrayList.add(overlayItem);
		}
		
		// Only add in if not already present
		List<Overlay> overlays = mapView.getOverlays();
		if (!overlays.contains(itemizedMarkerOverlay)) {
			overlays.add(itemizedMarkerOverlay);
		}
		if (!overlays.contains(circlesOverlay)) {
			overlays.add(circlesOverlay);
		}
	}
	
	
	protected void addTourLocations(MapView mapView)
	{
		for (int i=0; i<itemizedMarkerOverlayArrayList.size(); i++) {
			
			OverlayItem overlay_item = itemizedMarkerOverlayArrayList.get(i);
			itemizedMarkerOverlay.addOverlay(overlay_item);
			
			GeoPoint geo_point = circlesOverlayGeoPoints.get(i);
			int radius = circlesOverlayRadii.get(i);
			circlesOverlay.addCircleData(geo_point, radius);
		}

		mapView.invalidate();
	}
	
	protected void removeTourLocations(MapView mapView)
	{	
		for (int i=0; i<itemizedMarkerOverlayArrayList.size(); i++) {
			
			OverlayItem overlay_item = itemizedMarkerOverlayArrayList.get(i);
			itemizedMarkerOverlay.removeOverlay(overlay_item);
		}
		
		circlesOverlay.reset();
		
		mapView.invalidate();
		
	}
	
	protected ArrayList<HashMap<String,String>> parseXMLToHashMapArrayList(String xml_filename,String lock_on_element)
	{
		ArrayList<HashMap<String,String>> hashmap_list = null;
		
		try {
			
			// set up XML parser
			SAXParserFactory spf = SAXParserFactory.newInstance();
			SAXParser sp = spf.newSAXParser();
			XMLReader xr = sp.getXMLReader();

			// Create handler that maps XML Tags within <Location> to a hashmap 
			XMLToHashmapHandler xmlTourHandler = new XMLToHashmapHandler(lock_on_element);
			xr.setContentHandler(xmlTourHandler);
			
			File tour_file = new File(xml_filename);
			FileInputStream fis = new FileInputStream(tour_file);
			
			InputSource is = new InputSource(fis);
			xr.parse(is);
			
			// get the HashMap List that results from parsing the file
			hashmap_list = xmlTourHandler.getHashmapList();
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return hashmap_list;
	}
	public ArrayList<TipLocation> loadTour(String tour_filename)
	{
		ArrayList<HashMap<String,String>> location_hashmap_list 
			= parseXMLToHashMapArrayList(tour_filename,"Location");

		if (location_hashmap_list != null) {
			Log.d("TippleActivity::loadTour()", "XML Tour File Parsed");
		}
		
		ArrayList<TipLocation> locations = new ArrayList<TipLocation>();
		
		for (int i=0;  i<location_hashmap_list.size(); i++) {
			HashMap<String,String> location_hashmap = location_hashmap_list.get(i);
			
			TipLocation location = new TipLocation(location_hashmap);
			locations.add(location);
		}
		
		return locations;
	}
	
	protected ArrayList<TipLocation> loadUserTrail(String log_filename)
	{
		ArrayList<HashMap<String,String>> logentry_hashmap_list 
			= parseXMLToHashMapArrayList(log_filename,"LogEntry");

		ArrayList<TipLocation> locations = new ArrayList<TipLocation>();
		
		if (logentry_hashmap_list != null) {
			Log.d("TippleActivity::loadUserTrail()", "XML User Activity Log File Parsed");
			

			for (int i=0;  i<logentry_hashmap_list.size(); i++) {
				HashMap<String,String> logentry_hashmap = logentry_hashmap_list.get(i);
				
				HashMap<String,String> location_hashmap = new HashMap<String,String>();
				location_hashmap.put("longitude",logentry_hashmap.get("longitude"));
				location_hashmap.put("latitude",logentry_hashmap.get("latitude"));
				location_hashmap.put("radius",logentry_hashmap.get("accuracy"));
				location_hashmap.put("title",logentry_hashmap.get("action"));
				location_hashmap.put("text",logentry_hashmap.get("message"));
				
				TipLocation location = new TipLocation(location_hashmap);
				locations.add(location);
			}
		}
		
		return locations;
	}
	
	
	protected String SyncSynthesizeToFile(String text, String id, String root_utterance)
	{
		String audio_dir = TippleActivity.audioDirectory;
		File tts_file = new File(audio_dir, id+"-"+root_utterance+"-TTS.wav");
    	String tts_filename = tts_file.getAbsolutePath(); 	
    	
    	if (tts_file.exists()) {
    		// already synthesized this
    		return tts_filename;
    	}
    	
		// Rendering the supplied text of the id+utterance derived WAV file
    	HashMap<String, String> hashRender = new HashMap<String, String>();
    	hashRender.put(TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID, text);
    	
    	String UTTERANCE = root_utterance+"-utterance";
    	
    	hashRender.put(TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID, UTTERANCE);
    	UtteranceWaitListener listener = new UtteranceWaitListener(UTTERANCE);
    	tts_.setOnUtteranceCompletedListener(listener);
    	
    	 tts_.synthesizeToFile(text, hashRender, tts_filename);
    	 
    	try {
    		listener.waitForComplete();
    	} 
    	catch (Exception e) {
    		e.printStackTrace();
    		return null;
    	}
    	
    	return tts_filename;
	}
	
	boolean genPrevVisited = true;
	
	public boolean textToSpeechToLocation(TipLocation tip_location, boolean visited_previously) 
	{
		if (genPrevVisited) {
			prevVisitedFilename = SyncSynthesizeToFile("Previously visited.","Previously-visited","cue");
			System.err.println("** Prev visisted file = " + prevVisitedFilename);
			genPrevVisited = false;
		}
		
		String title = tip_location.getTitle();
		String text = tip_location.getText();
		String id = tip_location.getID();
		
		String title_filename = SyncSynthesizeToFile(title,id,"title");
		String text_filename = SyncSynthesizeToFile(text,id,"text");
	 
		if ((title_filename!=null) && (text_filename!=null)) {
			String audio_dir = TippleActivity.audioDirectory;
			File ding_file = new File(audio_dir, "DingExtended.wav");
			String ding_filename = ding_file.getAbsolutePath();
			
			if (audio_player_view_.addAudio(id)) {
				TippleActivity.log.optMessage("Queue Audio",title);
				audio_player_view_.queueAudio(title_filename,id,title,visited_previously);
				audio_player_view_.queueAudio(text_filename,id,text);
				audio_player_view_.queueAudio(ding_filename,id,"[Audio Bing]");
				
				audio_player_view_.updateTextViewIfHeadIsNewLocation();
			}
		}
    	
    	return true;
	}
	
	
	public static void stopOverlayThreads()
	{
	   	 if (circlesOverlay!=null) { 
	        	circlesOverlay.stop();
	        	circlesOverlay = null;
	        }
	       
	        if (itemizedMarkerOverlay!=null) {
	       	 itemizedMarkerOverlay.stop();
	        	itemizedMarkerOverlay = null;
	        }

	        if (routeOverlay!=null) {
	       	 routeOverlay.stop();
	        	routeOverlay = null;
	        }

	        if (routeMarkerOverlay!=null) {
	       	 routeMarkerOverlay.stop();
	        	routeMarkerOverlay = null;
	        }
	}
    private static class UtteranceWaitListener implements OnUtteranceCompletedListener {
        private boolean mIsComplete = false;
        private final String mExpectedUtterance;

        public UtteranceWaitListener(String expectedUtteranceId) {
            mExpectedUtterance = expectedUtteranceId;
        }

        public void onUtteranceCompleted(String utteranceId) {
            if (mExpectedUtterance.equals(utteranceId)) {
                synchronized(this) {
                    mIsComplete = true;	
                    notify();
                }
            }
        }
        
        public boolean waitForComplete() throws InterruptedException {
            if (mIsComplete) {	
                return true;
            }

            synchronized (this) {
                int TTS_SPEECH_MAX_WAIT_TIME = 5* 1000;
            	wait(TTS_SPEECH_MAX_WAIT_TIME);
                return mIsComplete;
            }
        }
        
    }    
}
